package com.ff.weijinxiao.controller;


import com.ff.weijinxiao.domain.*;
import com.ff.weijinxiao.jwt.JwtHelper;
import com.ff.weijinxiao.service.*;
import com.ff.weijinxiao.util.JsonTool;
import com.ff.weijinxiao.util.RandomUtil;
import com.ff.weijinxiao.util.ResultTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 客户还款接口
 */
@RestController
@RequestMapping("api/customerpayback")
@Api(value = "客户还款controller", tags = {"客户还款接口"})
public class CustomerPayBackController {

    @Resource
    private CustomerPayBackService customerPayBackService;

    @Resource
    private UserInfoService userInfoService;

    @Resource
    private CUserService cUserService;

    @Resource
    private CustomerService customerService;

    @Resource
    private AccountBookService accountBookService;


    public UserInfo getUserInfoByToken(HttpServletRequest request) {
        String token = request.getHeader("token");
        if (StringUtils.isBlank(token)) {
            return null;
        }
        String openId = JwtHelper.parseToken(token);
        if (StringUtils.isBlank(openId)) {
            return null;
        }

        CUser cUser = cUserService.findByOpenId(openId);
        if (null == cUser) {
            return null;
        }
        return cUser.getUserInfo();
    }

    @PostMapping("/update")
    @ApiOperation(value = "更新客户还款信息", notes = "更新客户还款信息")
    public Map<String, Object> updateCustomerPayBack(@RequestBody CustomerPayBack customerPayBack, HttpServletRequest request) {
        try {
            customerPayBackService.saveCustomerPayBack(customerPayBack);
            return ResultTool.genSuccessResult("修改成功");
        } catch (Exception e) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_2, "修改失败:" + e.getMessage());
        }
    }

    @PostMapping("/create")
    @ApiOperation(value = "创建客户还款信息", notes = "创建客户还款信息")
    public Map<String, Object> saveCustomerPayBack(@RequestBody CustomerPayBack customerPayBack, HttpServletRequest request) {
        try {
            customerPayBack.setIsDelete(0);
            customerPayBackService.saveCustomerPayBack(customerPayBack);
            return ResultTool.genSuccessResult("创建成功");
        } catch (Exception e) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_2, "创建失败:" + e.getMessage());
        }
    }


    @PostMapping("/delete")
    @ApiOperation(value = "删除客户还款信息", notes = "删除客户还款信息")
    public Map<String, Object> deleteCustomerPayBack(@ApiParam(required = true, value = "客户还款id") @RequestBody String jsonId) {
        Integer id = JsonTool.parseJsonIdStr(jsonId);
        if (id < 0) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "id参数错误");
        }
        CustomerPayBack customerPayBack = customerPayBackService.findById(id);
        if (null == customerPayBack) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "客户还款不存在");
        }
        customerPayBack.setIsDelete(1);
        customerPayBackService.saveCustomerPayBack(customerPayBack);
        return ResultTool.genSuccessResult("删除成功");
    }

    @GetMapping("/findbyid")
    @ApiOperation(value = "根据id查询客户还款信息", notes = "根据id查询客户还款信息")
    public Map<String, Object> findById(@ApiParam(required = true, value = "客户还款id") @RequestParam String id,
                                        HttpServletRequest request) {

        Map<String, Object> result = new HashMap<>();
        try {
            CustomerPayBack customerPayBack = customerPayBackService.findById(Integer.valueOf(id));
            if (null != customerPayBack) {
                result.put("data", customerPayBack);
                result.put("message", "查询成功");
                result.put("code", ResultTool.SuccessCode);
            } else {
                result.put("message", "查询失败，没有结果");
                result.put("code", ResultTool.ErrorCode_1);
            }

        } catch (Exception e) {
            e.printStackTrace();
            result.put("message", "查询失败" + e.getMessage());
            result.put("code", ResultTool.ErrorCode_2);
        }

        return result;
    }


    @GetMapping("/list")
    @ApiOperation(value = "查询客户还款列表", notes = "查询客户还款列表")
    public Map<String, Object> findList(@ApiParam(required = false, value = "客户还款名字") @RequestParam(required = false) String phone,
                                        HttpServletRequest request) {

        Map<String, Object> result = new HashMap<>();
        Map<String, Object> param = new HashMap<>();
        param.put("GT_id", 0);
        param.put("EQ_isDelete", 0);
        if (null != phone && phone.length() > 0) {
            param.put("LIKE_phone", phone);
        }
        List<CustomerPayBack> list = customerPayBackService.findList(param);
        if (null != list && list.size() > 0) {
            result.put("data", list);
            result.put("total", list.size());
            result.put("message", "查询成功");
            result.put("code", 10000);
        } else {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "没有查询到数据");
        }

        return result;
    }

    @GetMapping("/listpage")
    @ApiOperation(value = "查询客户还款分页列表", notes = "查询客户还款分页列表")
    public Map<String, Object> findListPage(@ApiParam(required = false, value = "客户还款名字") @RequestParam(required = false) String phone,
                                            @ApiParam(required = false, value = "页码 1开始") @RequestParam(required = false) Integer pageNo,
                                            @ApiParam(required = false, value = "每页显示条数 默认10") @RequestParam(required = false) Integer pageSize) {

        Map<String, Object> result = new HashMap<>();
        Map<String, Object> param = new HashMap<>();
        if (null == pageNo) {
            pageNo = 1;
        } else if (pageNo < 1) {
            pageNo = 1;
        }
        if (null == pageSize) {
            pageSize = 10;
        }
        param.put("GT_id", 0);
        param.put("EQ_isDelete", 0);
        if (null != phone && phone.length() > 0) {
            param.put("LIKE_phone", phone);
        }
        Page<CustomerPayBack> list = customerPayBackService.findPage(pageNo, pageSize, param);
        if (null != list && list.getTotalElements() > 0) {
            List<CustomerPayBack> data = list.getContent();
            for (CustomerPayBack customerPayBack : data) {
                UserInfo userInfo = userInfoService.findByUid(customerPayBack.getUid());
                if (null != userInfo) {
                    customerPayBack.setUserName(userInfo.getWechatName());
                }
            }
            result.put("data", list);
            result.put("message", "分页查询成功");
            result.put("code", 10000);
        } else {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "没有查询到数据");
        }

        return result;
    }

    @PostMapping("/wxapi/update")
    @ApiOperation(value = "更新客户还款信息", notes = "更新客户还款信息")
    public Map<String, Object> wxupdateCustomerPayBack(@RequestBody CustomerPayBack customerPayBack, HttpServletRequest request) {
        try {
            customerPayBackService.saveCustomerPayBack(customerPayBack);
            return ResultTool.genSuccessResult("修改成功");
        } catch (Exception e) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_2, "修改失败:" + e.getMessage());
        }
    }

    @PostMapping("/wxapi/create")
    @ApiOperation(value = "创建客户还款信息", notes = "创建客户还款信息")
    public Map<String, Object> wxsaveCustomerPayBack(@RequestBody CustomerPayBack customerPayBack,
            /*@ApiParam(required = true, value = "客户onlyid") @RequestParam(required = true) String customerOnlyId,*/
                                                     HttpServletRequest request) {
        try {
            Customer customer = customerService.findByOnlyID(customerPayBack.getCustomer().getOnlyID());
            if (null == customer) {
                return ResultTool.genFailResult(ResultTool.ErrorCode_2, "创建失败，客户不存在");
            }
            customerPayBack.setIsDelete(0);
            customerPayBack.setCustomer(customer);
            customerPayBack.setOnlyID(RandomUtil.genOnlyId());
            UserInfo userInfo = getUserInfoByToken(request);
            if (null != userInfo) {
                customerPayBack.setUid(userInfo.getUid());
            }
            AccountBook accountBook = accountBookService.getDefaultAccountBook(userInfo);
            customerPayBack.setAccountBook(accountBook);//设置默认账本
            customerPayBackService.saveCustomerPayBack(customerPayBack);
            return ResultTool.genSuccessResult("创建成功");
        } catch (Exception e) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_2, "创建失败:" + e.getMessage());
        }
    }

    @GetMapping("/wxapi/listpage")
    @ApiOperation(value = "查询客户还款分页列表", notes = "查询客户还款分页列表")
    public Map<String, Object> wxfindListPage(@ApiParam(required = false, value = "客户onlyid") @RequestParam(required = false) String customerOnlyId,
                                              @ApiParam(required = false, value = "客户电话") @RequestParam(required = false) String phone,
                                              @ApiParam(required = false, value = "客户姓名") @RequestParam(required = false) String name,
                                              @ApiParam(required = false, value = "页码 1开始") @RequestParam(required = false) Integer pageNo,
                                              @ApiParam(required = false, value = "每页显示条数 默认10") @RequestParam(required = false) Integer pageSize,
                                              HttpServletRequest request) {

        Map<String, Object> result = new HashMap<>();
        Map<String, Object> param = new HashMap<>();
        if (null == pageNo) {
            pageNo = 1;
        } else if (pageNo < 1) {
            pageNo = 1;
        }
        if (null == pageSize) {
            pageSize = 10;
        }
        param.put("GT_id", 0);
        param.put("EQ_isDelete", 0);
        if (null != phone && phone.length() > 0) {
            param.put("LIKE_customer.phone", phone);
        }
        if (null != name && name.length() > 0) {
            param.put("LIKE_customer.name", name);
        }
        UserInfo userInfo = getUserInfoByToken(request);
        if (null == userInfo) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "获取失败，未注册有效账号");
        }
        //param.put("EQ_uid", userInfo.getUid());
        param.put("EQ_accountBook.id", userInfo.getAccountBookId());
        if (StringUtils.isNotBlank(customerOnlyId)) {
            Customer customer = customerService.findByOnlyID(customerOnlyId);
            if (null == customer) {
                return ResultTool.genFailResult(ResultTool.ErrorCode_2, "获取失败，客户不存在");
            }
            param.put("EQ_customer.onlyID", customer.getOnlyID());
        }
        Page<CustomerPayBack> list = customerPayBackService.findPage(pageNo, pageSize, param);
        if (null != list && list.getTotalElements() > 0) {
            List<CustomerPayBack> data = list.getContent();
            for (CustomerPayBack customerPayBack : data) {
                customerPayBack.setUserName(userInfo.getWechatName());
            }
            result.put("data", list);
            result.put("message", "分页查询成功");
            result.put("code", 10000);
        } else {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "没有查询到数据");
        }

        return result;
    }


    @GetMapping("/wxapi/findbyid")
    @ApiOperation(value = "根据id查询客户还款信息", notes = "根据id查询客户还款信息")
    public Map<String, Object> wxfindById(@ApiParam(required = true, value = "客户还款id") @RequestParam String id,
                                          HttpServletRequest request) {

        Map<String, Object> result = new HashMap<>();
        try {
            CustomerPayBack customerPayBack = customerPayBackService.findById(Integer.valueOf(id));
            if (null != customerPayBack) {
                result.put("data", customerPayBack);
                result.put("message", "查询成功");
                result.put("code", ResultTool.SuccessCode);
            } else {
                result.put("message", "查询失败，没有结果");
                result.put("code", ResultTool.ErrorCode_1);
            }

        } catch (Exception e) {
            e.printStackTrace();
            result.put("message", "查询失败" + e.getMessage());
            result.put("code", ResultTool.ErrorCode_2);
        }

        return result;
    }


    @PostMapping("/wxapi/delete")
    @ApiOperation(value = "删除客户还款信息", notes = "删除客户还款信息")
    public Map<String, Object> wxdeleteCustomerPayBack(@ApiParam(required = true, value = "客户还款id") @RequestBody String jsonId) {
        Integer id = JsonTool.parseJsonIdStr(jsonId);
        if (id < 0) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "id参数错误");
        }
        CustomerPayBack customerPayBack = customerPayBackService.findById(id);
        if (null == customerPayBack) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "客户还款不存在");
        }
        customerPayBack.setIsDelete(1);
        customerPayBackService.saveCustomerPayBack(customerPayBack);
        return ResultTool.genSuccessResult("删除成功");
    }
}
