package com.ff.weijinxiao.controller;


import com.ff.weijinxiao.domain.*;
import com.ff.weijinxiao.jwt.JwtHelper;
import com.ff.weijinxiao.service.AccountBookService;
import com.ff.weijinxiao.service.CUserService;
import com.ff.weijinxiao.service.GoodsService;
import com.ff.weijinxiao.service.UserInfoService;
import com.ff.weijinxiao.util.JsonTool;
import com.ff.weijinxiao.util.RandomUtil;
import com.ff.weijinxiao.util.ResultTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 商品接口
 */
@RestController
@RequestMapping("api/goods")
@Api(value = "商品controller", tags = {"商品接口"})
public class GoodsController {

    @Resource
    private GoodsService goodsService;

    @Resource
    private UserInfoService userInfoService;

    @Resource
    private AccountBookService accountBookService;


    @PostMapping("/update")
    @ApiOperation(value = "更新商品信息", notes = "更新商品信息")
    public Map<String, Object> updateGoods(@RequestBody Goods goods, HttpServletRequest request) {
        try {
            goodsService.saveGoods(goods);
            return ResultTool.genSuccessResult("修改成功");
        } catch (Exception e) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_2, "修改失败:" + e.getMessage());
        }
    }

    @PostMapping("/create")
    @ApiOperation(value = "创建商品信息", notes = "创建商品信息")
    public Map<String, Object> saveGoods(@RequestBody Goods goods, HttpServletRequest request) {
        try {
            goodsService.saveGoods(goods);
            return ResultTool.genSuccessResult("创建成功");
        } catch (Exception e) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_2, "创建失败:" + e.getMessage());
        }
    }


    @PostMapping("/delete")
    @ApiOperation(value = "删除商品信息", notes = "删除商品信息")
    public Map<String, Object> deleteGoods(@ApiParam(required = true, value = "商品id") @RequestBody String jsonId) {
        Integer id = JsonTool.parseJsonIdStr(jsonId);
        if (id < 0) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "id参数错误");
        }
        Goods goods = goodsService.findById(id);
        if (null == goods) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "商品不存在");
        }
        goods.setIsDelete(1);
        goodsService.saveGoods(goods);
        return ResultTool.genSuccessResult("删除成功");
    }

    @GetMapping("/findbyid")
    @ApiOperation(value = "根据id查询商品信息", notes = "根据id查询商品信息")
    public Map<String, Object> findById(@ApiParam(required = true, value = "商品id") @RequestParam String id,
                                        HttpServletRequest request) {

        Map<String, Object> result = new HashMap<>();
        try {
            Goods goods = goodsService.findById(Integer.valueOf(id));
            if (null != goods) {
                result.put("data", goods);
                result.put("message", "查询成功");
                result.put("code", ResultTool.SuccessCode);
            } else {
                result.put("message", "查询失败，没有结果");
                result.put("code", ResultTool.ErrorCode_1);
            }

        } catch (Exception e) {
            e.printStackTrace();
            result.put("message", "查询失败" + e.getMessage());
            result.put("code", ResultTool.ErrorCode_2);
        }

        return result;
    }


    @GetMapping("/list")
    @ApiOperation(value = "查询商品列表", notes = "查询商品列表")
    public Map<String, Object> findList(@ApiParam(required = false, value = "商品名字") @RequestParam(required = false) String name,
                                        HttpServletRequest request) {

        Map<String, Object> result = new HashMap<>();
        Map<String, Object> param = new HashMap<>();
        param.put("GT_id", 0);
        param.put("EQ_isDelete", 0);
        if (null != name && name.length() > 0) {
            param.put("LIKE_phone", name);
        }
        List<Goods> list = goodsService.findList(param);
        if (null != list && list.size() > 0) {
            result.put("data", list);
            result.put("total", list.size());
            result.put("message", "查询成功");
            result.put("code", 10000);
        } else {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "没有查询到数据");
        }

        return result;
    }

    @GetMapping("/listpage")
    @ApiOperation(value = "查询商品分页列表", notes = "查询商品分页列表")
    public Map<String, Object> findListPage(@ApiParam(required = false, value = "商品名字") @RequestParam(required = false) String name,
                                            @ApiParam(required = false, value = "页码 1开始") @RequestParam(required = false) Integer pageNo,
                                            @ApiParam(required = false, value = "每页显示条数 默认10") @RequestParam(required = false) Integer pageSize) {

        Map<String, Object> result = new HashMap<>();
        Map<String, Object> param = new HashMap<>();
        if (null == pageNo) {
            pageNo = 1;
        } else if (pageNo < 1) {
            pageNo = 1;
        }
        if (null == pageSize) {
            pageSize = 10;
        }
        param.put("GT_id", 0);
        param.put("EQ_isDelete", 0);
        if (null != name && name.length() > 0) {
            param.put("LIKE_name", name);
        }
        Page<Goods> list = goodsService.findPage(pageNo, pageSize, param);
        if (null != list && list.getTotalElements() > 0) {
            List<Goods> data = list.getContent();
            for (Goods goods : data) {
                UserInfo userInfo = userInfoService.findByUid(goods.getUid());
                if (null != userInfo) {
                    goods.setUserName(userInfo.getWechatName());
                }
            }
            result.put("data", list);
            result.put("message", "分页查询成功");
            result.put("code", 10000);
        } else {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "没有查询到数据");
        }

        return result;
    }

    @Resource
    private CUserService cUserService;

    public UserInfo getUserInfoByToken(HttpServletRequest request) {
        String token = request.getHeader("token");
        if (StringUtils.isBlank(token)) {
            return null;
        }
        String openId = JwtHelper.parseToken(token);
        if (StringUtils.isBlank(openId)) {
            return null;
        }

        CUser cUser = cUserService.findByOpenId(openId);
        if (null == cUser) {
            return null;
        }
        return cUser.getUserInfo();
    }

    @PostMapping("/wxapi/create")
    @ApiOperation(value = "创建商品信息", notes = "创建商品信息")
    public Map<String, Object> wxCreateGoods(@RequestBody Goods goods, HttpServletRequest request) {
        try {
            Map<String, Object> result = new HashMap<>();
            UserInfo userInfo = getUserInfoByToken(request);
            if (null != userInfo) {
                goods.setUid(userInfo.getUid());
            }
            goods.setOnlyID(RandomUtil.genOnlyId());
            AccountBook accountBook = accountBookService.getDefaultAccountBook(userInfo);
            goods.setAccountBook(accountBook);//设置默认账本
            goods = goodsService.saveGoods(goods);
            result.put("goodsId", goods.getId());
            result.put("goodsOnlyId", goods.getOnlyID());
            result.put("code", 10000);
            result.put("message", "创建成功");
            return result;
        } catch (Exception e) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_2, "创建失败:" + e.getMessage());
        }
    }

    @GetMapping("/wxapi/list")
    @ApiOperation(value = "查询商品列表", notes = "查询商品列表")
    public Map<String, Object> wxfindList(@ApiParam(required = false, value = "商品名字") @RequestParam(required = false) String name,
                                          HttpServletRequest request) {

        Map<String, Object> result = new HashMap<>();
        UserInfo userInfo = getUserInfoByToken(request);
        if (null == userInfo) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "获取失败，未注册有效账号");
        }
        Map<String, Object> param = new HashMap<>();
        param.put("GT_id", 0);
        param.put("EQ_isDelete", 0);
        if (null != name && name.length() > 0) {
            param.put("LIKE_name", name);
        }
        //param.put("EQ_uid", userInfo.getUid());
        param.put("EQ_accountBook.id", userInfo.getAccountBookId());
        List<Goods> list = goodsService.findList(param);
        if (null != list && list.size() > 0) {
            result.put("data", list);
            result.put("total", list.size());
            result.put("message", "查询成功");
            result.put("code", 10000);
        } else {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "没有查询到数据");
        }

        return result;
    }

    @PostMapping("/wxapi/update")
    @ApiOperation(value = "更新商品信息", notes = "更新商品信息")
    public Map<String, Object> wxupdateGoods(@RequestBody Goods goods, HttpServletRequest request) {
        try {
            goodsService.saveGoods(goods);
            return ResultTool.genSuccessResult("修改成功");
        } catch (Exception e) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_2, "修改失败:" + e.getMessage());
        }
    }

}
