package com.ff.weijinxiao.controller;


import com.ff.weijinxiao.domain.WareHouse;
import com.ff.weijinxiao.domain.UserInfo;
import com.ff.weijinxiao.service.WareHouseService;
import com.ff.weijinxiao.service.UserInfoService;
import com.ff.weijinxiao.util.JsonTool;
import com.ff.weijinxiao.util.ResultTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 仓库接口
 */
@RestController
@RequestMapping("api/warehouse")
@Api(value = "仓库controller", tags = {"仓库接口"})
public class WareHouseController {

    @Resource
    private WareHouseService wareHouseService;

    @Resource
    private UserInfoService userInfoService;


    @PostMapping("/update")
    @ApiOperation(value = "更新仓库信息", notes = "更新仓库信息")
    public Map<String, Object> updateWareHouse(@RequestBody WareHouse wareHouse, HttpServletRequest request) {
        try {
            wareHouseService.saveWareHouse(wareHouse);
            return ResultTool.genSuccessResult("修改成功");
        } catch (Exception e) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_2, "修改失败:" + e.getMessage());
        }
    }

    @PostMapping("/create")
    @ApiOperation(value = "创建仓库信息", notes = "创建仓库信息")
    public Map<String, Object> saveWareHouse(@RequestBody WareHouse wareHouse, HttpServletRequest request) {
        try {
            wareHouseService.saveWareHouse(wareHouse);
            return ResultTool.genSuccessResult("创建成功");
        } catch (Exception e) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_2, "创建失败:" + e.getMessage());
        }
    }


    @PostMapping("/delete")
    @ApiOperation(value = "删除仓库信息", notes = "删除仓库信息")
    public Map<String, Object> deleteWareHouse(@ApiParam(required = true, value = "仓库id") @RequestBody String jsonId) {
        Integer id = JsonTool.parseJsonIdStr(jsonId);
        if (id < 0) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "id参数错误");
        }
        WareHouse wareHouse = wareHouseService.findById(id);
        if (null == wareHouse) {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "仓库不存在");
        }
        wareHouseService.deleteById(id);
        return ResultTool.genSuccessResult("删除成功");
    }

    @GetMapping("/findbyid")
    @ApiOperation(value = "根据id查询仓库信息", notes = "根据id查询仓库信息")
    public Map<String, Object> findById(@ApiParam(required = true, value = "仓库id") @RequestParam String id,
                                        HttpServletRequest request) {

        Map<String, Object> result = new HashMap<>();
        try {
            WareHouse wareHouse = wareHouseService.findById(Integer.valueOf(id));
            if (null != wareHouse) {
                result.put("data", wareHouse);
                result.put("message", "查询成功");
                result.put("code", ResultTool.SuccessCode);
            } else {
                result.put("message", "查询失败，没有结果");
                result.put("code", ResultTool.ErrorCode_1);
            }

        } catch (Exception e) {
            e.printStackTrace();
            result.put("message", "查询失败" + e.getMessage());
            result.put("code", ResultTool.ErrorCode_2);
        }

        return result;
    }


    @GetMapping("/list")
    @ApiOperation(value = "查询仓库列表", notes = "查询仓库列表")
    public Map<String, Object> findList(@ApiParam(required = false, value = "仓库名字") @RequestParam(required = false) String phone,
                                        HttpServletRequest request) {

        Map<String, Object> result = new HashMap<>();
        Map<String, Object> param = new HashMap<>();
        param.put("GT_id", 0);
        if (null != phone && phone.length() > 0) {
            param.put("LIKE_phone", phone);
        }
        List<WareHouse> list = wareHouseService.findList(param);
        if (null != list && list.size() > 0) {
            result.put("data", list);
            result.put("total", list.size());
            result.put("message", "查询成功");
            result.put("code", 10000);
        } else {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "没有查询到数据");
        }

        return result;
    }

    @GetMapping("/listpage")
    @ApiOperation(value = "查询仓库分页列表", notes = "查询仓库分页列表")
    public Map<String, Object> findListPage(@ApiParam(required = false, value = "仓库名字") @RequestParam(required = false) String tag,
                                            @ApiParam(required = false, value = "页码 1开始") @RequestParam(required = false) Integer pageNo,
                                            @ApiParam(required = false, value = "每页显示条数 默认10") @RequestParam(required = false) Integer pageSize) {

        Map<String, Object> result = new HashMap<>();
        Map<String, Object> param = new HashMap<>();
        if (null == pageNo) {
            pageNo = 1;
        } else if (pageNo < 1) {
            pageNo = 1;
        }
        if (null == pageSize) {
            pageSize = 10;
        }
        param.put("GT_id", 0);
        if (null != tag && tag.length() > 0) {
            param.put("LIKE_name", tag);
        }
        Page<WareHouse> list = wareHouseService.findPage(pageNo, pageSize, param);
        if (null != list && list.getTotalElements() > 0) {
            List<WareHouse> data = list.getContent();
            for(WareHouse wareHouse : data) {
                UserInfo userInfo = userInfoService.findByUid(wareHouse.getUid());
                if(null != userInfo) {
                    wareHouse.setUserName(userInfo.getWechatName());
                }
            }
            result.put("data", list);
            result.put("message", "分页查询成功");
            result.put("code", 10000);
        } else {
            return ResultTool.genFailResult(ResultTool.ErrorCode_1, "没有查询到数据");
        }

        return result;
    }

}
